#conditions-grid {
    margin: 20px;
}

#places-label {
    font-weight: bold;
}

#temperature-label {
    font-size: 32pt;
    font-weight: 900;
    margin-left: 16px;
}

#apparent-label {
    font-size: 9pt;
}

#weather-page-placeholder-title {
    font-weight: bold;
    font-size: 1.2em;
}

#loading-label {
    padding-top: 24px;
    font-size: 1.5em;
}

#attribution-label {
    font-size: small;
}

#conditions-grid *:backdrop {
    color: @theme_fg_color;
}

.content-view.cell {
    font-weight: bold;
}

#locations-list-box {
    border: 1px solid @borders;
}

.forecast-temperature-label {
    font-weight: bold;
    color: #c89009;
}

.forecast-low-temperature-label {
    font-weight: bold;
    color: #2174d9;
}

@define-color temp_chart_background_color @theme_base_color;
@define-color temp_chart_fill_color rgba(248, 228, 92, 0.5);
@define-color temp_chart_stroke_color rgba(246, 211, 45, 1.0);

#daily-forecast-frame {
    background-color: @temp_chart_background_color;
    border-radius: 9px;
}

#daily-forecast-frame > border {
    border-radius: 9px;
}

#hourly-forecast-frame {
    border-radius: 9px;
}

#hourly-forecast-frame > border {
    border-radius: 9px;
}

#updated-time-label {
    font-size: 9pt;
}

#attribution-label {
    font-size: 9pt;
    color: rgba(154, 153, 150, 1.0);
}

viewswitchertitle viewswitcher {
  margin-left: 100px;
  margin-right: 100px;
}

button.osd.circular {
  border-radius: 9999px;
  -gtk-outline-radius: 9999px;
  min-width: 24px;
  min-height: 24px;
}

button.osd.circular > image {
  padding: 12px;
}

.day-popover {
  background-color: @theme_base_color;
}

.small-label {
  font-size: 9pt;
}

.search-view scrolledwindow {
  background-color: @theme_selected_bg_color;
  color: @theme_selected_fg_color;
}

.search-view .large-title {
  font-weight: bold;
}
